﻿// Add ability to reference components from DlhSoft.ProjectData.GanttChart.LightWPF.Controls library (that partially shares namespaces and type names with DlhSoft.ProjectData.Core library).
// External alias identifier is provided by the assembly reference (Aliases property in Visual Studio Properties window).
extern alias pdgcc;

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DlhSoft.Windows.Data;
using DlhSoft.Windows.Services;
using System.IO;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace GanttChartExporterConfigurationSample
{
    class Program
    {
        static void Main(string[] args)
        {
            var items = new List<TaskItem>
            {
                new TaskItem { Tag = "Task 1" },
                new TaskItem { Tag = "Task 1.1", Indentation = 1, Start = DateTime.Today.Add(TimeSpan.Parse("08:00")), Finish = DateTime.Today.Add(TimeSpan.Parse("16:00")), CompletedFinish = DateTime.Today.Add(TimeSpan.Parse("12:00")), Assignments = new List<AssignmentItem> { new AssignmentItem { Resource = "Resource 1" } } },
                new TaskItem { Tag = "Task 1.2", Indentation = 1, Start = DateTime.Today.AddDays(1).Add(TimeSpan.Parse("08:00:00")), Finish = DateTime.Today.AddDays(2).Add(TimeSpan.Parse("16:00:00")), Assignments = new List<AssignmentItem> { new AssignmentItem { Resource = "Resource 1" }, new AssignmentItem { Resource = "Resource 2" } } },
                new TaskItem { Tag = "Task 2" },
                new TaskItem { Tag = "Task 2.1", Indentation = 1, Start = DateTime.Today.Add(TimeSpan.Parse("08:00")), Finish = DateTime.Today.AddDays(2).Add(TimeSpan.Parse("12:00")) },
                new TaskItem { Tag = "Task 2.2", Indentation = 1 },
                new TaskItem { Tag = "Task 2.2.1", Indentation = 2, Start = DateTime.Today.Add(TimeSpan.Parse("08:00:00")), Finish = DateTime.Today.AddDays(3).Add(TimeSpan.Parse("12:00:00")) },
                new TaskItem { Tag = "Task 2.2.2", Indentation = 2, Start = DateTime.Today.AddDays(4), IsMilestone = true },
                new TaskItem { Tag = "Task 3", Start = DateTime.Today.AddDays(3).Add(TimeSpan.Parse("12:00:00")), Finish = DateTime.Today.AddDays(6).Add(TimeSpan.Parse("14:00:00")), Assignments = new List<AssignmentItem> { new AssignmentItem { Resource = "Resource 1", AllocationUnits = 0.5 }, new AssignmentItem { Resource = "Resource 2", AllocationUnits = 0.75 } } },
                new TaskItem { Tag = "Task 4", Start = DateTime.Today.AddDays(6).Add(TimeSpan.Parse("08:00:00")), Finish = DateTime.Today.AddDays(6).Add(TimeSpan.Parse("16:00:00")), Assignments = new List<AssignmentItem> { new AssignmentItem { Resource = "Resource 1" } } },
                
                // You may certainly use CustomTaskItem type that inherits from TaskItem and adds supplemental properties such as Description.
                new CustomTaskItem { Tag = "Custom task", Start = DateTime.Today.Add(TimeSpan.Parse("08:00:00")), Finish = DateTime.Today.Add(TimeSpan.Parse("16:00:00")), Description = "Custom description" }
            };
            items[2].Predecessors.Add(new PredecessorItem { Item = items[1] });
            items[3].Predecessors.Add(new PredecessorItem { Item = items[0], DependencyType = DependencyType.StartStart });
            items[7].Predecessors.Add(new PredecessorItem { Item = items[4] });
            items[7].Predecessors.Add(new PredecessorItem { Item = items[6] });
            TaskManager taskManager = new TaskManager
            {
                Items = items,
                Schedule = Schedule.Standard
            };
            taskManager.Initialize();
            taskManager.LevelResources(DateTime.Today);

            byte[] imageBytes = GanttChartExporter.GetImageBytes(taskManager,
                // Specify control configuration actions.
                configuration: (control) =>
                {
                    // Configure grid columns.
                    var columns = control.Columns;
                    columns[0].MinWidth = 200;
                    columns[1].Header = "Start date";
                    columns[2].Header = "Finish date";
                    var effortColumn = new DataGridTextColumn { Header = "Effort (h)", Binding = new Binding("Effort.TotalHours") };
                    columns.Insert(3, effortColumn);

                    // Configure exporting custom columns referring CustomTaskItem properties such as Description using "Tag." prefix.
                    var descriptionColumn = new DataGridTextColumn { Header = "Description", Binding = new Binding("Tag.Description") };
                    columns.Add(descriptionColumn);

                    // Configure zoom level.
                    control.HourWidth = 3;

                    // Configure scales.
                    var majorScale = control.Scales[0];
                    majorScale.HeaderContentFormat = pdgcc::DlhSoft.Windows.Controls.TimeScaleTextFormat.MonthDay;
                    var minorScale = control.Scales[1];
                    minorScale.HeaderContentFormat = pdgcc::DlhSoft.Windows.Controls.TimeScaleTextFormat.DayOfWeekTwoInitials;

                    // Configure timeline page.
                    control.SetTimelinePage(taskManager.GetProjectStart().Date.AddDays(-3), taskManager.GetProjectFinish().Date.AddDays(14));

                    // Configure task bar appearance.
                    control.StandardBarFill = Brushes.LightBlue;
                    control.StandardBarCornerRadius = 0;
                },
                // Optionally, set other export parameters.
                resolution: 96 * 4, encoderType: typeof(JpegBitmapEncoder));

            var imageFileName = "Test.jpg";
            using (FileStream fileStream = new FileStream(imageFileName, FileMode.Create))
            {
                fileStream.Write(imageBytes, 0, imageBytes.Length);
            }
            Process.Start(imageFileName);
        }
    }
}
